from __future__ import annotations
import json
import string
import textwrap
import typing
import requests

from dataclasses import dataclass
from datetime import date, datetime
from .common import ArlulaObject
from .auth import Session
from .exception import ArlulaAPIException
from .orders import DetailedOrderResult
from .util import parse_rfc3339, calculate_price, remove_none, simple_indent

Polygon = typing.Union[typing.List[typing.List[typing.List[float]]], str]

class CenterPoint(ArlulaObject):
    data: dict
    long: float
    lat: float

    def __init__(self, data):
        self.data = data
        self.long = data["long"]
        self.lat = data["lat"]

    def dict(self) -> dict:
        return self.data

    def __str__(self) -> str:
        return f"Center: {self.long} {'E' if self.long < 0 else 'W'}, {self.lat} {'S' if self.lat < 0 else 'N'}"

class Percent(ArlulaObject):
    data: dict
    scene: float
    search: float

    def __init__(self, data):
        self.data = data
        self.scene = data["scene"]
        self.search = data["search"]

    def dict(self) -> dict:
        return self.data
    
    def __str__(self) -> str:
        return f"Coverage: {self.scene}% of scene, {self.search}% of search"

class Overlap(ArlulaObject):
    data: dict
    area: float
    percent: Percent
    polygon: typing.List[typing.List[typing.List[float]]]

    def __init__(self, data):
        self.data = data
        self.area = data["area"]
        self.percent = Percent(data["percent"])
        self.polygon = data["polygon"]

    def dict(self) -> dict:
        return self.data
    
    def __str__(self) -> str:
        return simple_indent(
        f"Overlap:\n"\
        f"Area: {self.area} sqkm\n"\
        f"{str(self.percent)}\n"\
        f"Geometry: {self.polygon}", 0, 2)

class License(ArlulaObject):
    data: dict
    name: str
    href: str
    loading_percent: float
    loading_amount: int

    def __init__(self, data):
        self.data = data
        self.name = data["name"]
        self.href = data["href"]
        self.loading_percent = data["loadingPercent"]
        self.loading_amount = data["loadingAmount"]

    def dict(self) -> dict:
        return self.data

    def __str__(self) -> str:
        return simple_indent(
        f"License ({self.href}):\n"\
        f"Name: {self.name}\n"\
        f"Loading: {self.loading_amount} US Cents + {self.loading_percent}%\n", 0, 2)

class Band(ArlulaObject):
    data: dict
    name: str
    id: str
    min: float
    max: float

    def __init__(self, data):
        self.data = data
        self.name = data["name"]
        self.id = data["id"]
        self.min = data["min"]
        self.max = data["max"]

    def centre(self) -> float:
        '''
            Get the band centre wavelength
        '''
        return (self.max - self.min)/2

    def width(self) -> float:
        '''
            Get the band width
        '''
        return self.max - self.min

    def dict(self) -> dict:
        return self.data

    def __str__(self) -> str:
        return simple_indent(
            f"Band ({self.id}):\n"\
            f"Name: {self.name}\n"\
            f"Bandwidth: {self.min}nm - {self.max}nm\n", 0, 2)

class Bundle(ArlulaObject):
    data: dict
    name: str
    key: str
    bands: typing.List[str]
    price: int

    def __init__(self, data):
        self.data = data
        self.name = data["name"]
        self.key = data["key"]
        self.bands = data["bands"]
        self.price = data["price"]

    def dict(self) -> dict:
        return self.data

    def __str__(self) -> str:
        bands = 'all' if len(self.bands) == 0 else '\n'.join(self.bands)
        return simple_indent(
            f"Bundle ({self.key}):\n"\
            f"Name: {self.name}\n"\
            f"Bands: {bands}\n"\
            f"Price: {self.price} US Cents\n", 0, 2)

class SearchResult(ArlulaObject):
    data: dict
    scene_id: str
    supplier: str
    platform: str
    date: datetime
    thumbnail: str
    cloud: float
    off_nadir: float
    gsd: float
    bands: typing.List[Band]
    area: float
    center: CenterPoint
    bounding: typing.List[typing.List[typing.List[float]]]
    overlap: Overlap
    fulfillment_time: float
    ordering_id: str
    bundles: typing.List[Bundle]
    license: typing.List[License]
    annotations: typing.List[str]

    def __init__(self, data):
        self.data = data
        self.scene_id = data["sceneID"]
        self.supplier = data["supplier"]
        self.platform = data["platform"]
        self.date = parse_rfc3339(data["date"])
        self.thumbnail = data["thumbnail"]
        self.cloud = data["cloud"]
        self.off_nadir = data["offNadir"]

        self.gsd = data["gsd"]

        self.bands = []
        self.bands += [Band(b) for b in data["bands"]]

        self.area = data["area"]
        self.center = CenterPoint(data["center"])
        self.bounding = data["bounding"]
        self.overlap = Overlap(data["overlap"])
        self.fulfillment_time = data["fulfillmentTime"]

        self.ordering_id = data["orderingID"]
            
        self.bundles = []
        self.bundles += [Bundle(b) for b in data["bundles"]]
        self.license = []
        self.license += [License(l) for l in data["license"]]

        self.annotations = []
        if "annotations" in data:
            self.annotations = data["annotations"]

    def calculate_price(self, license_href: string, bundle_key: string) -> int:
        '''
            Wrapper for util.calculate_price, returns price in US Cents. Raises error in the case of invalid license_name or bundle_key
        '''
        
        bundle = None
        license = None
        
        for b in self.bundles:
            if bundle.key == bundle_key:
                bundle = b
        
        for l in self.license:
            if license.href == license_href:
                license = l

        if bundle == None:
            raise ValueError("Invalid bundle_key")

        if license == None:
            raise ValueError("Invalid license_href")

        return calculate_price(bundle.price, license.loading_percent, license.loading_amount)

    def dict(self) -> dict:
        return self.data

    def __str__(self) -> str:

        bundles = simple_indent(''.join([str(b) for b in self.bundles]), 2, 2)
        bands = simple_indent(''.join([str(b) for b in self.bands]), 2, 2)
        license = simple_indent(''.join([str(l) for l in self.license]), 2, 2)
        return simple_indent(
            f"Result ({self.ordering_id}):\n"\
            f"Scene ID: {self.scene_id}\n"\
            f"Supplier: {self.supplier}\n"\
            f"Platform: {self.platform}\n"\
            f"Capture Date: {self.date.strftime('%Y-%m-%d')}\n"\
            f"Thumbnail URL: {self.thumbnail}\n"\
            f"Cloud Coverage: {self.cloud}%\n"\
            f"Off Nadir: {self.off_nadir} degrees\n"\
            f"Ground Sample Distance: {self.gsd} m\n"\
            f"Fulfillment Time: {self.fulfillment_time}\n"\
            f"Area: {self.area} sqkm\n"\
            f"{str(self.center)}\n"\
            f"Bounding: {self.bounding}\n"\
            f"{str(self.overlap)}"\
            f"Bands:\n"\
            f"{bands}"
            f"Bundles: \n"\
            f"{bundles}"
            f"License: \n"\
            f"{license}"
            f"Annotations: {', '.join(self.annotations)}\n", 0, 2)

class SearchResponse(ArlulaObject):
    data: dict
    state: string
    errors: typing.List[str]
    warnings: typing.List[str]
    results: typing.List[SearchResult]

    def __init__(self, data):
        self.data = data
        self.state = ""
        self.errors = []
        self.warnings = []
        self.results = []

        if "state" in data:
            self.state = data["state"]
        if "errors" in data:
            self.errors += data["errors"]
        if "warnings" in data:
            self.warnings += data["warnings"]
        if "results" in data:
            self.results = [SearchResult(e) for e in data["results"]]

    def __str__(self) -> str:
        s = ""
        for r in self.results:
            s += str(r)
        return s

    
    def dict(self) -> dict:
        return self.data

class SearchRequest():
    start: date
    gsd: float
    end: date
    lat: float
    long: float
    north: float
    south: float
    east: float
    west: float
    polygon: Polygon
    supplier: str
    off_nadir: float
    cloud: float

    def __init__(self, start: date,
            gsd: float,
            cloud: typing.Optional[float] = None,
            end: typing.Optional[date] = None,
            lat: typing.Optional[float] = None,
            long: typing.Optional[float] = None,
            north: typing.Optional[float] = None,
            south: typing.Optional[float] = None,
            east: typing.Optional[float] = None,
            west: typing.Optional[float] = None,
            supplier: typing.Optional[str] = None,
            off_nadir: typing.Optional[float] = None,
            polygon: Polygon = None):
        self.start = start
        self.cloud = cloud
        self.gsd = gsd
        self.end = end
        self.lat = lat
        self.long = long
        self.north = north
        self.south = south
        self.east = east
        self.west = west
        self.supplier = supplier
        self.off_nadir = off_nadir
        self.polygon = polygon

    def set_point_of_interest(self, lat: float, long: float) -> "SearchRequest":
        self.lat = lat
        self.long = long
        return self
    
    def set_area_of_interest(self, north: float, south: float, east: float, west: float) -> "SearchRequest":
        self.north = north
        self.south = south
        self.west = west
        self.east = east
        return self
    
    def set_polygon(self, polygon: Polygon) -> "SearchRequest":
        self.polygon = polygon
        return self

    def set_supplier(self, supplier: str) -> "SearchRequest":
        self.supplier = supplier
        return self

    def set_maximum_gsd(self, gsd: float) -> "SearchRequest":
        self.gsd = gsd
        return self

    def set_start(self, start: date) -> "SearchRequest":
        self.start = start
        return self

    def set_end(self, end: date) -> "SearchRequest":
        self.end = end
        return self
    
    def set_between_dates(self, start: date, end: date) -> "SearchRequest":
        self.start = start
        self.end = end
        return self
    
    def set_maximum_off_nadir(self, off_nadir: float) -> "SearchRequest":
        self.off_nadir = off_nadir
        return self

    def set_maximum_cloud_cover(self, cloud: float) -> "SearchRequest":
        self.cloud = cloud
        return self

    def valid_point_of_interest(self) -> bool:
        return self.lat != None and self.long != None

    def valid_area_of_interest(self) -> bool:
        return self.north != None and self.south != None and self.east != None and self.west != None
    
    def valid(self) -> bool:
        return (self.valid_area_of_interest() or self.valid_point_of_interest) and self.start != None and self.gsd != None
    
    def dict(self):
        param_dict = {
            "start": str(self.start) if self.start != None else None, 
            "end": str(self.end) if self.end != None else None,
            "gsd": self.gsd, "cloud": self.cloud,
            "lat": self.lat, "long": self.long,
            "north": self.north, "south": self.south, "east": self.east, 
            "west": self.west, "supplier": self.supplier, "off-nadir": self.off_nadir,
            "polygon": json.dumps(self.polygon) if isinstance(self.polygon, list) else self.polygon}

        query_params = {k: v for k, v in param_dict.items()
            if v is not None}

        return remove_none(query_params)

class OrderRequest(ArlulaObject):

    id: str
    eula: str
    bundle_key: str
    webhooks: typing.List[str]
    emails: typing.List[str]

    def __init__(self,
            id: str,
            eula: str,
            bundle_key: str,
            webhooks: typing.Optional[typing.List[str]] = [],
            emails: typing.Optional[typing.List[str]] = [],
            team: typing.Optional[str] = None):
        self.id = id
        self.eula = eula
        self.bundle_key = bundle_key
        self.webhooks = webhooks
        self.emails = emails
        self.team = team
    
    def add_webhook(self, webhook: str) -> "OrderRequest":
        self.webhooks.append(webhook)
        return self
    
    def set_webhooks(self, webhooks: typing.List[str]) -> "OrderRequest":
        self.webhooks = webhooks
        return self

    def add_email(self, email: str) -> "OrderRequest":
        self.emails.append(email)
        return self
    
    def set_emails(self, emails: typing.List[str]) -> "OrderRequest":
        self.emails = emails
        return self

    def set_team(self, team: str) -> "OrderRequest":
        self.team = team
        return self

    def valid(self) -> bool:
        return self.id != None and self.eula != None and self.bundle_key != None

    def dict(self):
        return remove_none({
            "id": self.id,
            "eula": self.eula,
            "bundleKey": self.bundle_key,
            "webhooks": self.webhooks,
            "emails": self.emails,
            "team": None if self.team == "" else self.team,
        })


class ArchiveAPI:
    '''
        Archive is used to interface with the Arlula Archive API
    '''

    def __init__(self,
                 session: Session):
        self.session = session
        self.url = self.session.baseURL + "/api/archive"

    def search(self, request: SearchRequest) -> SearchResponse:
        '''
            Search the Arlula imagery archive.
            Requires one of (lat, long) or (north, south, east, west).
        '''

        url = self.url+"/search"

        # Send request and handle responses
        response = requests.request(
            "GET", url,
            headers=self.session.header,
            params=request.dict())
        if response.status_code != 200:
            raise ArlulaAPIException(response)
        else:
            resp_data = json.loads(response.text)
            # Construct an instance of `SearchResponse`
            return SearchResponse(resp_data)

    def order(self, request: OrderRequest) -> DetailedOrderResult:
        '''
            Order from the Arlula imagery archive
        '''

        url = self.url + "/order"

        response = requests.request(
            "POST",
            url,
            data=json.dumps(request.dict()),
            headers=self.session.header)

        if response.status_code != 200:
            raise ArlulaAPIException(response)
        else:
            return DetailedOrderResult(json.loads(response.text))



import sys
import platform
import base64
import typing
import requests

from .exception import ArlulaSessionError

# Package Name
name = "arlulacore"

# User agent setting
sdk_version = "3.1.0"
py_version = sys.version.split(' ')[0]
os_version = platform.platform()
def_ua = "core-sdk " + \
    sdk_version + " python " + py_version + " OS " + os_version

# Expected API version
x_api_version = '2023-01'

class Session:
    '''
        Session handles authentication for the arlula API.
    '''

    def __init__(self,
                 key: str,
                 secret: str,
                 user_agent: typing.Optional[str] = def_ua,
                 url: typing.Optional[str] = "https://api.arlula.com",
                 test: typing.Optional[bool] = True,
                 ):
        # Encode the key and secret
        def atob(x): return x.encode('utf-8')
        self.token = base64.b64encode(atob(
            key + ':' + secret)).decode('utf-8')
        self.header = {
            'Authorization': "Basic "+self.token,
            'User-Agent': user_agent,
            'X-API-Version': x_api_version
        }
        if url is None:
            url = "https://api.arlula.com"
        self.baseURL = url
        if test:
            self.validate_creds()

    # Check the credentials are valid
    def validate_creds(self):
        url = self.baseURL+"/api/test"

        headers = self.header

        response = requests.request("GET", url, headers=headers)

        if response.status_code != 200:
            raise ArlulaSessionError(response.text)



import abc
import json

class ArlulaObject(abc.ABC):
    def __repr__(self):
        return str(['{}: {}'.format(attr, value) for attr, value in self.__dict__.items()])[1:-1].replace('\'', '')

    @abc.abstractmethod
    def dict(self):
        pass

    def format(self, format: str) -> str:
        if format == "json":
            return json.dumps(self.dict())
        if format == "text":
            return str(self)
        if format == "pretty-json":
            return json.dumps(self.dict(), indent=1)
        if format == "table":
            return str(self)



import requests

'''
    Custom Exception Class
'''

class ArlulaSessionError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return self.value

class ArlulaAPIException(Exception):
    def __init__(self, response: requests.Response):
        self.value = f"{response.status_code}: {response.text}"

    def __str__(self):
        return self.value


'''
    Objects specific to the orders api
'''

from datetime import datetime
import os
import typing
import json
import requests
import sys
import re

from .auth import Session
from .exception import ArlulaAPIException, ArlulaSessionError
from .common import ArlulaObject
from .util import parse_rfc3339, simple_indent

disposition_name_regex = re.compile(r"\"([\w\.]+)\"")

class Resource(ArlulaObject):
    data: dict
    id: str
    created_at: datetime
    updated_at: datetime
    order: str
    name: str
    type: str
    format: str
    roles: typing.List[str]
    size: int
    checksum: str

    def __init__(self, data):
        self.data = data
        self.id = data["id"]
        self.created_at = parse_rfc3339(data["createdAt"])
        self.updated_at = parse_rfc3339(data["updatedAt"])
        self.order = data["order"]
        self.name = data["name"]
        self.type = data["type"]
        self.format = data["format"]
        self.roles = data["roles"]
        self.size = data["size"]
        self.checksum = data["checksum"]

    def __str__(self) -> str:
        text = simple_indent(
            f"Resource ({self.id}):\n"\
            # f"Created At: {self.created_at.isoformat()}\n"\
            # f"Updated At: {self.updated_at.isoformat()}\n"\
            # f"Order ID: {self.order}\n"\
            f"Name: {self.name}\n"\
            f"Type: {self.type}\n"\
            f"Format: {self.format}\n"\
            f"Roles: {', '.join(self.roles)}\n"\
            f"Size: {self.size} Bytes\n"\
            f"Checksum: {self.checksum}\n", 0, 2)
        return text

    def dict(self):
        return self.data

class OrderResult(ArlulaObject):
    data: dict
    id: str
    created_at: datetime
    updated_at: datetime
    supplier: str
    ordering_id: str
    scene_id: str
    status: str
    total: int
    type: str
    expiration: typing.Optional[str]

    def __init__(self, data):
        self.data = data
        self.id = data["id"]
        self.created_at = parse_rfc3339(data["createdAt"])
        self.updated_at = parse_rfc3339(data["updatedAt"])
        self.supplier = data["supplier"]
        self.ordering_id = data["orderingID"]
        self.scene_id = data["sceneID"]
        self.status = data["status"]
        self.total = data["total"]
        self.type = data["type"]
        self.expiration = data["expiration"]

    def __str__(self) -> str:
        text = simple_indent(
            f"Order ({self.id}):\n"\
            # f"Created At: {self.created_at}\n"\
            # f"Updated At: {self.updated_at}\n"\
            f"Supplier: {self.supplier}\n"\
            # f"Ordering ID: {self.ordering_id}\n"\
            f"Scene ID: {self.scene_id}\n"\
            f"Status: {self.status}\n"\
            f"Total: {self.total}\n"\
            f"Type: {self.type}\n"\
            f"Expiration: {self.expiration}\n", 0, 2)
        return text

    def dict(self) -> dict:
        return self.data

class DetailedOrderResult(ArlulaObject):
    data: dict
    id: str
    created_at: datetime
    updated_at: datetime
    supplier: str
    ordering_id: str
    scene_id: str
    status: str
    total: int
    type: str
    expiration: typing.Optional[str]
    resources: typing.List[Resource]

    def __init__(self, data):
        self.data = data
        self.id = data["id"]
        self.created_at = parse_rfc3339(data["createdAt"])
        self.updated_at = parse_rfc3339(data["updatedAt"])
        self.supplier = data["supplier"]
        self.ordering_id = data["orderingID"]
        self.scene_id = data["sceneID"]
        self.status = data["status"]
        self.total = data["total"]
        self.type = data["type"]
        self.expiration = data["expiration"]
        self.resources = [Resource(x) for x in data["resources"]] if "resources" in data else []
    
    def __str__(self) -> dict:
        resources = simple_indent(''.join([str(r) for r in self.resources]), 2, 2)
        text = simple_indent(
            f"Detailed Order ({self.id}):\n"\
            # f"Created At: {self.created_at}\n"\
            # f"Updated At: {self.updated_at}\n"\
            f"Supplier: {self.supplier}\n"\
            # f"Ordering ID: {self.ordering_id}\n"\
            f"Scene ID: {self.scene_id}\n"\
            f"Status: {self.status}\n"\
            f"Total: {self.total}\n"\
            f"Type: {self.type}\n"\
            f"Expiration: {self.expiration}\n"\
            f"Resources:\n"\
            f"{resources}", 0, 2)
        return text

    def dict(self) -> dict:
        return self.data

class OrdersAPI:
    '''
        Orders is used to interface with the Arlula Orders API
    '''

    def __init__(self,
                 session: Session):
        self.session = session
        self.url = self.session.baseURL + "/api/order"

    def get(self,
            id: str) -> DetailedOrderResult:
        '''
            Get details on a prior order by it's id.
        '''

        url = self.url + "/get"

        querystring = {"id": id}

        response = requests.request(
            "GET",
            url,
            headers=self.session.header,
            params=querystring)

        if response.status_code != 200:
            raise ArlulaAPIException(response)
        else:
            return DetailedOrderResult(json.loads(response.text))

    def list(self) -> typing.List[OrderResult]:
        '''
            List all orders made
        '''

        url = self.url+"/list"

        response = requests.request(
            "GET",
            url,
            headers=self.session.header)

        if response.status_code != 200:
            raise ArlulaAPIException(response)
        else:
            return [OrderResult(r) for r in json.loads(response.text)]

    def download_order(self, id: str, directory: typing.Optional[str], suppress: typing.Optional[bool]=False) -> None:
        order = self.get(id)
        for r in order.resources:
            self.get_resource_as_file(r.id, suppress=suppress, directory=directory).close()

    def get_resource_as_file(self,
                     id: str,
                     filepath: typing.Optional[str] = None,
                     suppress: typing.Optional[bool] = False,
                     progress_generator: typing.Optional[typing.Generator[typing.Optional[float], None, None]] = None,
                     directory: typing.Optional[str] = None) -> typing.BinaryIO:
        '''
            Get a resource and stream it to the specified file. If supress is true, it will output extra information to standard output.
            If filename is not supplied, it will use the file specified by the supplier through the Content-Disposition header. If a filename
            is not supplied and a directory is then the default named file will be placed in the specified directory, otherwise it is ignored.
            This is recommended for large files. Returns the file, which must be closed. The returned file is seeked back to it's beginning.
        '''

        url = self.url + "/resource/get"

        
        querystring = {"id": id}
        if progress_generator is not None:
            next(progress_generator)

        # Stream response
        response = requests.request(
            "GET",
            url,
            headers=self.session.header,
            params=querystring,
            stream=True)
        
        if response.status_code != 200:
            raise ArlulaAPIException(response)

        if filepath is None:
            # As requests follows redirects, need to use the history
            content_disposition = response.history[0].headers.get("content-disposition")
            filename = disposition_name_regex.findall(content_disposition)[0]
            dir = directory or os.getcwd()
            filepath = os.path.join(dir, filename)
        f = open(filepath, "w+b")

        total = response.headers.get('content-length')


        if total is None:
            f.write(response.content)
        else:
            # Write the response in chunks
            # Chunk size is the larger of 0.1% of the filesize or 1MB
            downloaded = 0
            total = int(total)
            
            for data in response.iter_content(chunk_size=max(int(total/1000), 1024*1024)):
                downloaded += len(data)
                f.write(data)

                # Track progress of download
                done = int(50*downloaded/total)
                if not suppress:
                    sys.stdout.write('\r[{}{}]{:.2%}'.format(
                        '█' * done, '.' * (50-done), downloaded/total))
                    sys.stdout.flush()
                if progress_generator is not None:
                    progress_generator.send(downloaded/total)

        if not suppress:
            sys.stdout.write('\n')
            sys.stdout.write('download complete\n')

        # So the file is able to be read from the beginning.
        f.seek(0)
        
        return f

    def get_resource_as_memory(self, id: str) -> bytes:
        '''
            Get a resource. If filepath is specified, it will be streamed to that file. If filepath is omitted it will
            be stored in memory (not recommended for large files).
        '''
        url = self.url + "/resource/get"

        querystring = {"id": id}

        response = requests.request(
            "GET",
            url,
            headers=self.session.header,
            params=querystring)
        
        return response.content



from .auth import (
    Session,
)

from .archive import (
    CenterPoint,
    Percent,
    Overlap,
    License,
    Band,
    Bundle,
    SearchResult,
    SearchResponse,
    SearchRequest,
    OrderRequest,
    ArchiveAPI,
    Polygon,
)
from .orders import (
    OrderResult,
    OrdersAPI,
    DetailedOrderResult,
)

from .arlula import (
    ArlulaAPI,
)

from .common import (
    ArlulaObject,
)

from .exception import (
    ArlulaSessionError,
    ArlulaAPIException,
)

from .util import (
    parse_rfc3339,
    calculate_price,
)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='arlulacore',
    version='3.1.0',
    author="Arlula",
    author_email="tech@arlula.com",
    description="A package to facilitate access to the Arlula Imagery Marketplace API",

    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Arlula/python-core-sdk.git",
    packages=["arlulacore"],
    install_requires=['requests'],
    classifiers=[
        "Programming Language :: Python :: 3",
         "License :: OSI Approved :: MIT License",
         "Operating System :: OS Independent",
    ],
)



from .auth import Session
from .archive import ArchiveAPI
from .orders import OrdersAPI

class ArlulaAPI:
    '''
        Main class for Arlula API Calls. Contains an instance of the archive and orders APIs.
    '''

    def __init__(self, session: Session):
        self._archive = ArchiveAPI(session)
        self._orders = OrdersAPI(session)
    
    def archiveAPI(self) -> ArchiveAPI:
        '''
            Returns the archive api instance, used for searching and ordering imagery.
        '''
        return self._archive

    def ordersAPI(self) -> OrdersAPI:
        '''
            Returns the orders api instance, used for maintaining orders and getting resources.
        '''
        return self._orders



import math
import re
import requests

from datetime import datetime, timezone, timedelta
from arlulacore.exception import ArlulaSessionError

__date_rx__ = re.compile(r"^(?P<year>\d{4})-(?P<month>\d\d)-(?P<day>\d\d)[Tt](?P<hour>\d\d):(?P<minute>\d\d):(?P<second>\d\d)(?:\.(?P<sec_frac>\d+))?(?P<offset>(?:[zZ]|(?P<offset_sign>[+-])(?P<offset_hour>\d{2}):(?P<offset_minute>\d{2})))$")

def remove_none(d: dict) -> dict:
    return {k: v for k, v in d.items() if v is not None}

def get_error(resp: requests.Response):
    return ArlulaSessionError(f"{resp.status_code}: {resp.text}")

def simple_indent(s: str, first_amount: int, following_amount: int) -> str:
    lines = s.splitlines()
    out = [""]*len(lines)

    for i, l in enumerate(lines):
        if i == 0:
            out[i] = first_amount*' ' + l
        else:
            out[i] = following_amount*' ' + l
    return '\n'.join(out) + '\n'

def parse_rfc3339(dt_str: str) -> datetime:
    try:
        result = re.search(__date_rx__, dt_str)

        if result.lastindex < 7 or result.lastindex > 11:
            return None

        sec_frac_int = 0

        sec_frac = result["sec_frac"]
        if sec_frac is not None:
            sec_frac_str = sec_frac[:6] if len(sec_frac) > 6 else sec_frac
            sec_frac_int = int(sec_frac_str)*(10**(6 - len(sec_frac_str)))

        tz = timezone(timedelta())

        if not (result["offset"] == "z" or result["offset"] == "Z"):

            offset_hour = int(result["offset_hour"])
            offset_minute = int(result["offset_minute"])

            if result["offset_sign"] == "-":
                offset_minute *= -1
                offset_hour *= -1
            
            tz = timezone(timedelta(hours=offset_hour, minutes=offset_minute))

        return datetime(
            int(result["year"]),
            int(result["month"]),
            int(result["day"]),
            int(result["hour"]),
            int(result["minute"]),
            int(result["second"]),
            sec_frac_int,
            tz
        )
    except:
        return None

def calculate_price(bundle_price: int, loading_percent: float, loading_amount: int) -> int:
    """
    Calculates the price for a scene in US Cents.

    Parameters:
        bundle_price (int): The price of the bundle (taken from a Bundle)
        loading_percent (float): The percentage loading of the license (taken from a License)
        loading_amoutn (int): The flat loading of the license (taken from a License)
    
    Returns:
        price (int): The price in US Cents
    """
    price = bundle_price * (1 + (loading_percent/100))
    price = price + loading_amount
    return math.ceil(price/100)*100
